@@ -27,33 +27,126 @@
 #include "bitlbee.h"
 #include "account.h"
 
-account_t *account_add( irc_t *irc, int protocol, char *user, char *pass )
+account_t *account_add( irc_t *irc, struct prpl *prpl, char *user, char *pass )
 {
 	account_t *a;
+	set_t *s;
 	
 	if( irc->accounts )
 	{
 		for( a = irc->accounts; a->next; a = a->next );
-		a = a->next = g_new0 ( account_t, 1 );
+		a = a->next = g_new0( account_t, 1 );
 	}
 	else
 	{
 		irc->accounts = a = g_new0 ( account_t, 1 );
 	}
 	
-	a->protocol = protocol;
+	a->prpl = prpl;
 	a->user = g_strdup( user );
 	a->pass = g_strdup( pass );
+	a->auto_connect = 1;
 	a->irc = irc;
 	
+	s = set_add( &a->set, "auto_connect", "true", set_eval_account, a );
+	s->flags |= ACC_SET_NOSAVE;
+	
+	s = set_add( &a->set, "auto_reconnect", "true", set_eval_bool, a );
+	
+	s = set_add( &a->set, "password", NULL, set_eval_account, a );
+	s->flags |= ACC_SET_NOSAVE;
+	
+	s = set_add( &a->set, "username", NULL, set_eval_account, a );
+	s->flags |= ACC_SET_NOSAVE | ACC_SET_OFFLINE_ONLY;
+	set_setstr( &a->set, "username", user );
+	
+	a->nicks = g_hash_table_new_full( g_str_hash, g_str_equal, g_free, g_free );
+	
+	/* This function adds some more settings (and might want to do more
+	   things that have to be done now, although I can't think of anything. */
+	if( prpl->init )
+		prpl->init( a );
+	
 	return( a );
 }
 
+char *set_eval_account( set_t *set, char *value )
+{
+	account_t *acc = set->data;
+	
+	/* Double-check: We refuse to edit on-line accounts. */
+	if( set->flags & ACC_SET_OFFLINE_ONLY && acc->ic )
+		return NULL;
+	
+	if( strcmp( set->key, "username" ) == 0 )
+	{
+		g_free( acc->user );
+		acc->user = g_strdup( value );
+		return value;
+	}
+	else if( strcmp( set->key, "password" ) == 0 )
+	{
+		g_free( acc->pass );
+		acc->pass = g_strdup( value );
+		return NULL;	/* password shouldn't be visible in plaintext! */
+	}
+	else if( strcmp( set->key, "server" ) == 0 )
+	{
+		g_free( acc->server );
+		if( *value )
+		{
+			acc->server = g_strdup( value );
+			return value;
+		}
+		else
+		{
+			acc->server = NULL;
+			return g_strdup( set->def );
+		}
+	}
+	else if( strcmp( set->key, "auto_connect" ) == 0 )
+	{
+		if( !is_bool( value ) )
+			return NULL;
+		
+		acc->auto_connect = bool2int( value );
+		return value;
+	}
+	
+	return NULL;
+}
+
 account_t *account_get( irc_t *irc, char *id )
 {
 	account_t *a, *ret = NULL;
+	char *handle, *s;
 	int nr;
 	
+	/* This checks if the id string ends with (...) */
+	if( ( handle = strchr( id, '(' ) ) && ( s = strchr( handle, ')' ) ) && s[1] == 0 )
+	{
+		struct prpl *proto;
+		
+		*s = *handle = 0;
+		handle ++;
+		
+		if( ( proto = find_protocol( id ) ) )
+		{
+			for( a = irc->accounts; a; a = a->next )
+				if( a->prpl == proto &&
+				    a->prpl->handle_cmp( handle, a->user ) == 0 )
+					ret = a;
+		}
+		
+		/* Restore the string. */
+		handle --;
+		*handle = '(';
+		*s = ')';
+		
+		if( ret )
+			return ret;
+	}
+	
 	if( sscanf( id, "%d", &nr ) == 1 && nr < 1000 )
 	{
 		for( a = irc->accounts; a; a = a->next )
@@ -65,7 +158,7 @@ account_t *account_get( irc_t *irc, char *id )
 	
 	for( a = irc->accounts; a; a = a->next )
 	{
-		if( g_strcasecmp( id, proto_name[a->protocol] ) == 0 )
+		if( g_strcasecmp( id, a->prpl->name ) == 0 )
 		{
 			if( !ret )
 				ret = a;
@@ -91,7 +184,7 @@ void account_del( irc_t *irc, account_t *acc )
 	for( a = irc->accounts; a; a = (l=a)->next )
 		if( a == acc )
 		{
-			if( a->gc ) return; /* Caller should have checked, accounts still in use can't be deleted. */
+			if( a->ic ) return; /* Caller should have checked, accounts still in use can't be deleted. */
 			
 			if( l )
 			{
@@ -102,6 +195,11 @@ void account_del( irc_t *irc, account_t *acc )
 				irc->accounts = a->next;
 			}
 			
+			while( a->set )
+				set_del( &a->set, a->set->key );
+			
+			g_hash_table_destroy( a->nicks );
+			
 			g_free( a->user );
 			g_free( a->pass );
 			if( a->server ) g_free( a->server );
@@ -115,39 +213,22 @@ void account_del( irc_t *irc, account_t *acc )
 
 void account_on( irc_t *irc, account_t *a )
 {
-	struct aim_user *u;
-	
-	if( a->gc )
+	if( a->ic )
 	{
 		/* Trying to enable an already-enabled account */
 		return;
 	}
 	
-	if( proto_prpl[a->protocol]->login == NULL )
-	{
-		irc_usermsg( irc, "Support for protocol %s is not included in this BitlBee", proto_name[a->protocol] );
-		return;
-	}
-	
 	cancel_auto_reconnect( a );
 	
-	u = g_new0 ( struct aim_user, 1 );
-	u->irc = irc;
-	u->protocol = a->protocol;
-	strncpy( u->username, a->user, sizeof( u->username ) - 1 );
-	strncpy( u->password, a->pass, sizeof( u->password ) - 1 );
-	if( a->server) strncpy( u->proto_opt[0], a->server, sizeof( u->proto_opt[0] ) - 1 );
-	
-	a->gc = (struct gaim_connection *) u; /* Bit hackish :-/ */
 	a->reconnect = 0;
-	
-	proto_prpl[a->protocol]->login( u );
+	a->prpl->login( a );
 }
 
 void account_off( irc_t *irc, account_t *a )
 {
-	account_offline( a->gc );
-	a->gc = NULL;
+	imc_logout( a->ic, FALSE );
+	a->ic = NULL;
 	if( a->reconnect )
 	{
 		/* Shouldn't happen */
