@@ -26,26 +26,32 @@
 #ifndef __CONF_H
 #define __CONF_H
 
-typedef enum runmode { RUNMODE_DAEMON, RUNMODE_INETD } runmode_t;
+typedef enum runmode { RUNMODE_DAEMON, RUNMODE_FORKDAEMON, RUNMODE_INETD } runmode_t;
 typedef enum authmode { AUTHMODE_OPEN, AUTHMODE_CLOSED, AUTHMODE_REGISTERED } authmode_t;
 
 typedef struct conf
 {
 	char *iface;
-	signed int port;
+	char *port;
 	int nofork;
 	int verbose;
 	runmode_t runmode;
 	authmode_t authmode;
-	char *password;
+	char *auth_pass;
+	char *oper_pass;
 	char *hostname;
 	char *configdir;
+	char *plugindir;
+	char *pidfile;
 	char *motdfile;
+	char *primary_storage;
+	char **migrate_storage;
 	int ping_interval;
 	int ping_timeout;
+	char *user;
 } conf_t;
 
-conf_t *conf_load( int argc, char *argv[] );
+G_GNUC_MALLOC conf_t *conf_load( int argc, char *argv[] );
 void conf_loaddefaults( irc_t *irc );
 
 #endif
