@@ -32,17 +32,19 @@
 #define IRC_LOGIN_TIMEOUT 60
 #define IRC_PING_STRING "PinglBee"
 
-#define UMODES "ais"
+#define UMODES "abisw"
+#define UMODES_PRIV "Ro"
 #define CMODES "nt"
 #define CMODE "t"
 #define UMODE "s"
 
 typedef enum
 {
-	USTATUS_OFFLINE,
-	USTATUS_AUTHORIZED,
-	USTATUS_LOGGED_IN,
-	USTATUS_IDENTIFIED
+	USTATUS_OFFLINE = 0,
+	USTATUS_AUTHORIZED = 1,
+	USTATUS_LOGGED_IN = 2,
+	USTATUS_IDENTIFIED = 4,
+	USTATUS_SHUTDOWN = 8
 } irc_status_t;
 
 typedef struct channel
@@ -58,7 +60,6 @@ typedef struct irc
 	int pinging;
 	char *sendbuffer;
 	char *readbuffer;
-	int quit;
 
 	int sentbytes;
 	time_t oldtime;
@@ -87,39 +88,40 @@ typedef struct irc
 	GHashTable *userhash;
 	GHashTable *watches;
 	struct __NICK *nicks;
-	struct help *help;
 	struct set *set;
 
-	GIOChannel *io_channel;
 	gint r_watch_source_id;
 	gint w_watch_source_id;
 	gint ping_source_id;
 } irc_t;
 
 #include "user.h"
-#include "nick.h"
+// #include "nick.h"
 
 extern GSList *irc_connection_list;
 
 irc_t *irc_new( int fd );
+void irc_abort( irc_t *irc, int immed, char *format, ... ) G_GNUC_PRINTF( 3, 4 );
 void irc_free( irc_t *irc );
 
-int irc_exec( irc_t *irc, char **cmd );
-int irc_process( irc_t *irc );
-int irc_process_line( irc_t *irc, char *line );
+void irc_exec( irc_t *irc, char **cmd );
+void irc_process( irc_t *irc );
+char **irc_parse_line( char *line );
+char *irc_build_line( char **cmd );
 
 void irc_vawrite( irc_t *irc, char *format, va_list params );
-void irc_write( irc_t *irc, char *format, ... );
-void irc_write_all( int now, char *format, ... );
-void irc_reply( irc_t *irc, int code, char *format, ... );
-G_MODULE_EXPORT int irc_usermsg( irc_t *irc, char *format, ... );
+void irc_write( irc_t *irc, char *format, ... ) G_GNUC_PRINTF( 2, 3 );
+void irc_write_all( int now, char *format, ... ) G_GNUC_PRINTF( 2, 3 );
+void irc_reply( irc_t *irc, int code, char *format, ... ) G_GNUC_PRINTF( 3, 4 );
+G_MODULE_EXPORT int irc_usermsg( irc_t *irc, char *format, ... ) G_GNUC_PRINTF( 2, 3 );
 char **irc_tokenize( char *buffer );
 
 void irc_login( irc_t *irc );
+int irc_check_login( irc_t *irc );
 void irc_motd( irc_t *irc );
 void irc_names( irc_t *irc, char *channel );
 void irc_topic( irc_t *irc, char *channel );
-void irc_umode_set( irc_t *irc, char *who, char *s );
+void irc_umode_set( irc_t *irc, char *s, int allow_priv );
 void irc_who( irc_t *irc, char *channel );
 void irc_spawn( irc_t *irc, user_t *u );
 void irc_join( irc_t *irc, user_t *u, char *channel );
@@ -128,13 +130,14 @@ void irc_kick( irc_t *irc, user_t *u, char *channel, user_t *kicker );
 void irc_kill( irc_t *irc, user_t *u );
 void irc_invite( irc_t *irc, char *nick, char *channel );
 void irc_whois( irc_t *irc, char *nick );
-int irc_away( irc_t *irc, char *away );
+void irc_setpass( irc_t *irc, const char *pass ); /* USE WITH CAUTION! */
 
 int irc_send( irc_t *irc, char *nick, char *s, int flags );
 int irc_privmsg( irc_t *irc, user_t *u, char *type, char *to, char *prefix, char *msg );
 int irc_msgfrom( irc_t *irc, char *nick, char *msg );
 int irc_noticefrom( irc_t *irc, char *nick, char *msg );
 
-int buddy_send_handler( irc_t *irc, user_t *u, char *msg, int flags );
+void buddy_send_handler( irc_t *irc, user_t *u, char *msg, int flags );
+struct groupchat *irc_chat_by_channel( irc_t *irc, char *channel );
 
 #endif
