@@ -28,11 +28,9 @@
 #define TYPING_NOTIFICATION_MESSAGE "\r\r\rBEWARE, ME R TYPINK MESSAGE!!!!\r\r\r"
 #define GROUPCHAT_SWITCHBOARD_MESSAGE "\r\r\rME WANT TALK TO MANY PEOPLE\r\r\r"
 
-#ifdef _WIN32
-#define debug 
+#ifdef DEBUG
+#define debug( text... ) imcb_log( ic, text );
 #else
-#define debug( text... ) irc_usermsg( IRC, text );
-#undef debug
 #define debug( text... )
 #endif
 
@@ -56,7 +54,7 @@
 
 struct msn_data
 {
-	struct gaim_connection *gc;
+	struct im_connection *ic;
 	
 	int fd;
 	struct msn_handler_data *handler;
@@ -65,13 +63,18 @@ struct msn_data
 	
 	GSList *msgq;
 	GSList *switchboards;
-	int buddycount;
+	int sb_failures;
+	time_t first_sb_failure;
+	
 	const struct msn_away_state *away_state;
+	int buddycount;
+	int groupcount;
+	char **grouplist;
 };
 
 struct msn_switchboard
 {
-	struct gaim_connection *gc;
+	struct im_connection *ic;
 	
 	int fd;
 	gint inp;
@@ -85,7 +88,7 @@ struct msn_switchboard
 	
 	GSList *msgq;
 	char *who;
-	struct conversation *chat;
+	struct groupchat *chat;
 };
 
 struct msn_away_state
@@ -142,17 +145,19 @@ GSList *msn_connections;
 GSList *msn_switchboards;
 
 /* ns.c */
-void msn_ns_connected( gpointer data, gint source, GaimInputCondition cond );
+gboolean msn_ns_connected( gpointer data, gint source, b_input_condition cond );
 
 /* msn_util.c */
-int msn_write( struct gaim_connection *gc, char *s, int len );
-int msn_logged_in( struct gaim_connection *gc );
-int msn_buddy_list_add( struct gaim_connection *gc, char *list, char *who, char *realname );
-int msn_buddy_list_remove( struct gaim_connection *gc, char *list, char *who );
-void msn_buddy_ask( struct gaim_connection *gc, char *handle, char *realname );
+int msn_write( struct im_connection *ic, char *s, int len );
+int msn_logged_in( struct im_connection *ic );
+int msn_buddy_list_add( struct im_connection *ic, char *list, char *who, char *realname );
+int msn_buddy_list_remove( struct im_connection *ic, char *list, char *who );
+void msn_buddy_ask( struct im_connection *ic, char *handle, char *realname );
 char *msn_findheader( char *text, char *header, int len );
 char **msn_linesplit( char *line );
 int msn_handler( struct msn_handler_data *h );
+char *msn_http_encode( const char *input );
+void msn_msgq_purge( struct im_connection *ic, GSList **list );
 
 /* tables.c */
 const struct msn_away_state *msn_away_state_by_number( int number );
@@ -162,11 +167,11 @@ const struct msn_status_code *msn_status_by_number( int number );
 
 /* sb.c */
 int msn_sb_write( struct msn_switchboard *sb, char *s, int len );
-struct msn_switchboard *msn_sb_create( struct gaim_connection *gc, char *host, int port, char *key, int session );
-struct msn_switchboard *msn_sb_by_handle( struct gaim_connection *gc, char *handle );
-struct msn_switchboard *msn_sb_by_id( struct gaim_connection *gc, int id );
-struct msn_switchboard *msn_sb_spare( struct gaim_connection *gc );
+struct msn_switchboard *msn_sb_create( struct im_connection *ic, char *host, int port, char *key, int session );
+struct msn_switchboard *msn_sb_by_handle( struct im_connection *ic, char *handle );
+struct msn_switchboard *msn_sb_by_chat( struct groupchat *c );
+struct msn_switchboard *msn_sb_spare( struct im_connection *ic );
 int msn_sb_sendmessage( struct msn_switchboard *sb, char *text );
-void msn_sb_to_chat( struct msn_switchboard *sb );
+struct groupchat *msn_sb_to_chat( struct msn_switchboard *sb );
 void msn_sb_destroy( struct msn_switchboard *sb );
-void msn_sb_connected( gpointer data, gint source, GaimInputCondition cond );
+gboolean msn_sb_connected( gpointer data, gint source, b_input_condition cond );
