@@ -45,7 +45,6 @@ int chat_modfirst(aim_session_t *sess, aim_module_t *mod);
 int locate_modfirst(aim_session_t *sess, aim_module_t *mod);
 int general_modfirst(aim_session_t *sess, aim_module_t *mod);
 int ssi_modfirst(aim_session_t *sess, aim_module_t *mod);
-int popups_modfirst(aim_session_t *sess, aim_module_t *mod);
 int icq_modfirst(aim_session_t *sess, aim_module_t *mod);
 
 int aim_genericreq_n(aim_session_t *, aim_conn_t *conn, guint16 family, guint16 subtype);
@@ -107,7 +106,6 @@ typedef struct aim_snac_s {
 } aim_snac_t;
 
 void aim_initsnachash(aim_session_t *sess);
-aim_snacid_t aim_newsnac(aim_session_t *, aim_snac_t *newsnac);
 aim_snacid_t aim_cachesnac(aim_session_t *sess, const guint16 family, const guint16 type, const guint16 flags, const void *data, const int datalen);
 aim_snac_t *aim_remsnac(aim_session_t *, aim_snacid_t id);
 void aim_cleansnacs(aim_session_t *, int maxage);
@@ -151,7 +149,6 @@ struct snacgroup {
 	struct snacgroup *next;
 };
 
-#ifdef FAIM_NEED_CONN_INTERNAL
 struct snacpair {
 	guint16 group;
 	guint16 subtype;
@@ -171,7 +168,6 @@ struct rateclass {
 	struct snacpair *members;
 	struct rateclass *next;
 };
-#endif /* FAIM_NEED_CONN_INTERNAL */
 
 /*
  * This is inside every connection.  But it is a void * to anything
@@ -193,9 +189,8 @@ int aim_putcap(aim_bstream_t *bs, guint32 caps);
 int aim_cachecookie(aim_session_t *sess, aim_msgcookie_t *cookie);
 aim_msgcookie_t *aim_uncachecookie(aim_session_t *sess, guint8 *cookie, int type);
 aim_msgcookie_t *aim_mkcookie(guint8 *, int, void *);
-aim_msgcookie_t *aim_checkcookie(aim_session_t *, const unsigned char *, const int);
+aim_msgcookie_t *aim_checkcookie(aim_session_t *sess, const unsigned char *, const int);
 int aim_freecookie(aim_session_t *sess, aim_msgcookie_t *cookie);
-int aim_msgcookie_gettype(int reqclass);
 int aim_cookie_free(aim_session_t *sess, aim_msgcookie_t *cookie);
 
 int aim_extractuserinfo(aim_session_t *sess, aim_bstream_t *bs, aim_userinfo_t *);
