@@ -29,16 +29,27 @@
 static void query_display( irc_t *irc, query_t *q );
 static query_t *query_default( irc_t *irc );
 
-query_t *query_add( irc_t *irc, struct gaim_connection *gc, char *question, void *yes, void *no, void *data )
+query_t *query_add( irc_t *irc, struct im_connection *ic, char *question, void *yes, void *no, void *data )
 {
 	query_t *q = g_new0( query_t, 1 );
 	
-	q->gc = gc;
+	q->ic = ic;
 	q->question = g_strdup( question );
 	q->yes = yes;
 	q->no = no;
 	q->data = data;
 	
+	if( strchr( irc->umode, 'b' ) != NULL )
+	{
+		char *s;
+		
+		/* At least for the machine-parseable version, get rid of
+		   newlines to make "parsing" easier. */
+		for( s = q->question; *s; s ++ )
+			if( *s == '\r' || *s == '\n' )
+				*s = ' ';
+	}
+	
 	if( irc->queries )
 	{
 		query_t *l = irc->queries;
@@ -51,7 +62,7 @@ query_t *query_add( irc_t *irc, struct gaim_connection *gc, char *question, void
 		irc->queries = q;
 	}
 	
-	if( g_strcasecmp( set_getstr( irc, "query_order" ), "lifo" ) == 0 || irc->queries == q )
+	if( g_strcasecmp( set_getstr( &irc->set, "query_order" ), "lifo" ) == 0 || irc->queries == q )
 		query_display( irc, q );
 	
 	return( q );
@@ -85,7 +96,7 @@ void query_del( irc_t *irc, query_t *q )
 	g_free( q );
 }
 
-void query_del_by_gc( irc_t *irc, struct gaim_connection *gc )
+void query_del_by_conn( irc_t *irc, struct im_connection *ic )
 {
 	query_t *q, *n, *def;
 	int count = 0;
@@ -95,7 +106,7 @@ void query_del_by_gc( irc_t *irc, struct gaim_connection *gc )
 	
 	while( q )
 	{
-		if( q->gc == gc )
+		if( q->ic == ic )
 		{
 			n = q->next;
 			query_del( irc, q );
@@ -110,7 +121,7 @@ void query_del_by_gc( irc_t *irc, struct gaim_connection *gc )
 	}
 	
 	if( count > 0 )
-		serv_got_crap( gc, "Flushed %d unanswered question(s) for this connection.", count );
+		imcb_log( ic, "Flushed %d unanswered question(s) for this connection.", count );
 	
 	q = query_default( irc );
 	if( q && q != def )
@@ -126,15 +137,20 @@ void query_answer( irc_t *irc, query_t *q, int ans )
 		q = query_default( irc );
 		disp = 1;
 	}
-	//Using irc_usermsg instead of serv_got_crap because \x02A is a char too, so a SPACE is needed.	
 	if( ans )
 	{
-		serv_got_crap( q->gc, "Accepted: %s", q->question );
+		if( q->ic )
+			imcb_log( q->ic, "Accepted: %s", q->question );
+		else
+			irc_usermsg( irc, "Accepted: %s", q->question );
 		q->yes( NULL, q->data );
 	}
 	else
 	{
-		serv_got_crap( q->gc, "Rejected: %s", q->question );
+		if( q->ic )
+			imcb_log( q->ic, "Rejected: %s", q->question );
+		else
+			irc_usermsg( irc, "Rejected: %s", q->question );
 		q->no( NULL, q->data );
 	}
 	q->data = NULL;
@@ -147,9 +163,9 @@ void query_answer( irc_t *irc, query_t *q, int ans )
 
 static void query_display( irc_t *irc, query_t *q )
 {
-	if( q->gc )
+	if( q->ic )
 	{
-		serv_got_crap( q->gc, "New request: %s\nYou can use the \2yes\2/\2no\2 commands to accept/reject this request.", q->question );
+		imcb_log( q->ic, "New request: %s\nYou can use the \2yes\2/\2no\2 commands to accept/reject this request.", q->question );
 	}
 	else
 	{
@@ -161,7 +177,7 @@ static query_t *query_default( irc_t *irc )
 {
 	query_t *q;
 	
-	if( g_strcasecmp( set_getstr( irc, "query_order" ), "fifo" ) == 0 )
+	if( g_strcasecmp( set_getstr( &irc->set, "query_order" ), "fifo" ) == 0 )
 		q = irc->queries;
 	else
 		for( q = irc->queries; q && q->next; q = q->next );
